{Copyright 1989,1990 by
Stephen A. Theberge.
Bignum.pas version 2.02}
unit bignum;

interface

   function plus(s1,s2: string; data_ok: boolean) :string;
   function minus(s1,s2: string; data_ok: boolean): string;
   function times(s1,s2: string; data_ok: boolean): string;
   function divide(s1,s2: string; data_ok,no_decimals: boolean): string;
   function fac(s1: string): string;
   function square(s1: string): string;
   function root(s1: string):string;

implementation

   const
      big = 10000; {Version 2.01 addition}

   type
      anarray = array[128..255] of longint; {Version 2.01 change}
   var
      t1,t2,r: anarray; {term1, term2, result BCD math on s1 and s2}
      cnt,dpos1,dpos2,spos1,spos2,l1,l2,borrow: byte;
      carry: longint;{Version 2.01 change}
      temp: string;
      validstring: boolean;

function verify(var s1: string): boolean;
   var
      check,at: string;
      index,negcount,deccount: byte;
   begin
      check :='1234567890-.';
      negcount := 0;
      deccount := 0;
      if length(s1) = 1 then
         begin
            if s1 ='.' then
               begin
                  verify :=false;
                  exit;
               end;
            if s1 ='-' then
               begin
                  verify := false;
                  exit;
               end;
         end;

      if s1='' then
         begin
            verify := false;
            exit;
         end;

      for index :=1 to length(s1) do
         begin
            at := s1[index];
            if pos(at,check) = 0 then
               begin
                  verify := false;
                  exit;
               end;
            if at ='.' then
               deccount := deccount + 1;
            if at = '-' then
               negcount := negcount +1;
         end; {for}

         if (deccount>1) or (negcount>1) then
            begin
               verify :=false;
               exit;
            end
         else
            verify :=true;

         if (pos('-',s1) >1 ) then
            verify :=false
         else
            verify :=true;

      end; {verify}

procedure strip(var s1: string);
   var
      cnt: byte;
      tmp: string;
   begin
      cnt := pos('.',s1);
      if cnt = 0 then
         exit
      else
         begin
            if cnt = 1 then
               s1 := copy(s1,2,length(s1))
            else
               if cnt = length(s1) then
                  s1 := copy(s1,1,length(s1) -1)
               else
                  begin
                     tmp := copy(s1,1,cnt-1);
                     tmp := tmp+copy(s1,cnt+1,length(s1));
                     s1 := tmp;
                  end
          end;
   end;{strip}

procedure decicheck(var s1,s2: string);
   begin
      temp := 'good';
      if (dpos1 or dpos2) >= 1 then
         begin
            if dpos1 = 0 then
               begin
                  if length(s1)=255 then
                     begin
                        temp :='bad';
                        exit;
                     end
                  else
                     s1 :=s1+'.';
               end;

            if dpos2=0 then
               begin
                  if length(s2)=255 then
                     begin
                        temp :='bad';
                        exit;
                     end
                  else
                     s2 := s2+'.';
               end;

            while pos('.',s1) <> pos('.',s2) do
               begin
                  if pos('.',s1) < pos('.',s2) then
                     begin
                        if length(s1) = 255 then
                           begin
                              temp :='bad';
                              exit;
                           end
                        else
                           s1 := '0'+s1;
                     end
                  else
                     begin
                        if length(s2) = 255 then
                           begin
                              temp := 'bad';
                              exit;
                           end
                        else
                           s2 :='0'+s2;
                     end;
               end; {while}

            while length(s1) < length(s2) do
               s1 := s1+'0'; {pad until decimal points are alligned}
            while length(s2) < length(s1) do
               s2 := s2+'0';
         end {decimal point cases}
      else
         exit;{no need to go on, as there are no decimal points}

            dpos1 := pos('.',s1);
            dpos2 := dpos1;

      strip(s1);{remove decimal point}
      strip(s2);{for now}
      l1 := length(s1);
      l2 := l1;
end;{decicheck}


procedure offleadzero(var s: string);
   var
      zeros: boolean;
   begin
      zeros := true;
      while (length(s) >1) and (zeros) do
         begin
            if s[1]='0' then
               s := copy(s,2,length(s))
            else
               zeros := false;
         end;{while}
      if s = '.' then
         s := '0';
end;{offleadzero}

procedure offtrailzero(var s: string);
   var
      take: string;
   begin
      if pos('.',s)>0 then {remove trailing zeroes if decimal #}
         begin
            take := s[length(s)];
            while (length(s)>1) and (take='0') do
               begin
                  s := copy(s,1,length(s)-1);
                  take := s[length(s)];
               end;
         end;
   if s='.'then
      s :='0';
end;{offtrailzero}

procedure place(var s1: string; var t1: anarray; var l1: byte);
{version 2.01 change.  Numbers placed in array by groups of four as longints.}
{This allows more efficeint math later--speeds up BIGNUM.}

   var
      c: byte;
      newl1: byte;
      offset: byte;
      cnt: real;
      tmp: string[9];
      code: integer;

   begin
      cnt := frac(l1 / 4) * 4;
      c := round(cnt);
      tmp := copy(s1,1,c);
      newl1 := (l1 div 4) + 1;
      if tmp <>'' then
         val(tmp,t1[255 - newl1+1],code);

      offset := 2;
      while c< l1 do
         begin
            t1[255 - newl1 + (c div 4) + offset] :=
            ((ord(s1[c+1]) - 48) * 1000) +
            ((ord(s1[c+2]) - 48) * 100) +
            ((ord(s1[c+3]) - 48) * 10) +
            ((ord(s1[c+4]) - 48));
            c := c + 4;
         end;
   end;{place}

function adjust(r:anarray; cnt,dpos: byte): string;
{Version 2.01 change.  Groups of "4" put back into string.  Padding to show}
{0=0000, 1=0001, etc.}
   var
      tmp1: string;
      tmp2: string;
      cn: byte;
      tm: longint;
   begin
      tmp2 := '';
      cn := cnt;

      str(r[cn],tmp1);
      tmp2 := tmp2 + tmp1;
      cn := cn + 1;

      while cn <> 0 do
         begin
            str(r[cn],tmp1);
            if r[cn] < 1000 then
               tmp1 := '0' + tmp1;
            if r[cn] < 100 then
               tmp1 := '0' + tmp1;
            if r[cn] < 10 then
               tmp1 := '0' + tmp1;{Padding condition to allign.}
            tmp2 := tmp2 + tmp1;
            cn := cn + 1;
         end;{while}

   if dpos >0 then
      if (length(tmp2) = 255) and (dpos=0) then
         tmp2 := 'bad'
      else
         tmp2 := copy(tmp2,1,dpos-1) + '.' + copy(tmp2,dpos,length(tmp2));
   adjust := tmp2;

end;{adjust}

function plus(s1,s2: string; data_ok: boolean): string;
   var
      temp2: string;
   begin
      if not data_ok then
         begin
            validstring := verify(s1);
            if not(validstring) then
               begin
                  plus :='bad';
                  exit;
               end;
            validstring := verify(s2);
            if not(validstring) then
               begin
                  plus :='bad';
                  exit;
               end;
         end;

      if (s1 = '0') or (s2 = '0') then
         if s1<>'0' then
            begin
               plus := s1;{if adding 0, return either s1 or s2}
               exit;
            end
         else
            begin
               plus := s2;
               exit;
            end;

      dpos1 := pos('.',s1);
      spos1 := pos('-',s1);
      spos2 := pos('-',s2);
      dpos2 := pos('.',s2);
      l1    := length(s1);
      l2    := length(s2);
      if ((spos1 = 1 ) and(spos2 = 0)) or((spos1 = 0) and(spos2 = 1)) then
      {Adding a positive and negative is like subtracting }
         begin
            if spos1 = 1 then
               begin
                  s1 := copy(s1,2,length(s1));
                  plus := minus(s2,s1,true);
                  exit;
               end
            else
               begin
                  s2 := copy(s2,2,length(s2));
                  plus := minus(s1,s2,true);
                  exit;
               end;
         end;

            if (spos1 or spos2) = 1 then
               begin
                  s1 := copy(s1,2,length(s1));
                  s2 := copy(s2,2,length(s2));{take away signs for now}
               end;


         if (dpos1>0) or (dpos2>0) then
            decicheck(s1,s2);{handle and allign decimal points}
         if temp = 'bad' then
            begin
               plus :='bad';
               exit;
            end;

         if (spos1=1) and(dpos2=0) then
            begin {shorten length due to sign bit unless decimal points}
               l1:=l1-1;
               l2:=l2-1;
            end;

         while l1<l2 do
            begin
               s1 :='0'+s1;{pad with leading zeros}
               l1 := l1+1;
            end;

         while l2<l1 do
            begin
               s2 := '0'+s2;
               l2 := l2+1;
            end;

         if (l1=0) or (l2=0) then
            begin
               plus :='bad';
               exit;
            end;

         place(s1,t1,l1);{place string in math array}
         place(s2,t2,l2);

         cnt :=255;
         carry :=0;

         l1 := ((l1 -1 ) div 4) + 1;{Version 2.01 addition}

         while (255 - l1) < cnt do
            begin
               r[cnt] := t1[cnt]+t2[cnt]+carry;
               if r[cnt]>=big then
                  begin
                     carry := 1;
                     r[cnt] := r[cnt] - big;{Big is 2.01 addition}
                  end
               else
                  carry := 0;

               cnt := cnt-1;
            end;{while}

            if (cnt =191) and(r[192] >999) then {Version 2.01 add.}
               begin {Is 256th digit occupied?  Longint Groupint consideration.}
                  plus :='bad';
                  exit;
               end;

            if carry = 1 then
               begin
                  r[cnt] := 1;
                  if dpos1 >0 then
                     dpos1 := dpos1 + 1;
               end
            else
               cnt := cnt +1;

            if (dpos1 >0) and (carry=0) then
               begin
                  if t1[cnt] > t2[cnt] then
                     begin
                        str(t1[cnt],temp);
                        str(r[cnt],temp2);
                        if length(temp) < length(temp2) then
                           dpos1 := dpos1 + 1;
                     end
                  else
                     begin
                        str(t2[cnt],temp);
                        str(r[cnt],temp2);
                        if length(temp) < length(temp2) then
                           dpos1 := dpos1 + 1;
                     end;
                  carry :=1;
                  {while (s1[carry] = '0') and (carry <dpos1) do
                     begin
                        carry := carry + 1;
                        dpos1 := dpos1 - 1;
                     end;}
               end;

            temp := adjust(r,cnt,dpos1);
            if temp[1] = '0' then
               offleadzero(temp);
            if (pos('.',temp)>0) and (temp[length(temp)]='0') then
               offtrailzero(temp);
            if (spos1 and spos2) =1 then
               begin
                  if (length(temp) = 255) and (pos('.',temp)=0) then
                     plus :='bad'
                  else
                     plus :='-'+temp;
               end
            else
               plus :=temp;

end;{plus}

function minus(s1,s2: string; data_ok: boolean): string;
   var
      switched,borrowed: boolean;{determine if number order was switched}
   begin
   switched := false;
   borrowed := false;
   if not data_ok then
      begin
         validstring := verify(s1);
         if not(validstring) then
            begin
               minus :='bad';
               exit;
            end;
         validstring := verify(s2);
         if not(validstring) then
            begin
               minus :='bad';
               exit;
            end;

     end;
     dpos1 := pos('.',s1);
     spos1 := pos('-',s1);
     spos2 := pos('-',s2);
     dpos2 := pos('.',s2);
     l1    := length(s1);
     l2    := length(s2);

     if s2='0' then
        begin
           minus :=s1;
           exit;
        end;

     if (spos1 or spos2) = 1 then {check for conditions like addition}
        begin
           if (spos1 =1) and (spos2 = 0) then
              begin
                 s1 := copy(s1,2,length(s1));
                 temp := plus(s1,s2,true);
                 if (length(temp)=255) or(temp='bad') then
                   begin
                      minus :='bad';
                      exit;
                   end
                 else
                    begin
                       minus :='-'+temp;{negative minus a positive}
                       exit;
                    end;
              end;

              if (spos1 = 0) and(spos2 =1) then
                 begin
                    s2 := copy(s2,2,length(s2));
                    temp := plus(s1,s2,true);
                    if (length(temp)=255) or(temp='bad') then
                       begin
                          minus :='bad';
                          exit;
                       end
                    else
                       begin
                          minus := temp;{positive minus a negative}
                          exit;
                       end;
                 end;
              end;

           if s1=s2 then
              begin
                 minus :='0';{subtraction of equal (1-1) gives zero}
                 exit;
              end;

           if spos1=1 then
              s1 := copy(s1,2,length(s1));
           if spos2=1 then
              s2 := copy(s2,2,length(s2));

           if (dpos1>0) or (dpos2>0) then
              decicheck(s1,s2);

           if temp='bad' then
              begin
                 minus := temp;
                 exit;
              end;

           if (spos1=1) and(dpos2=0) then
            begin {shorten length due to sign bit unless decimal points}
               l1:=l1-1;
               l2:=l2-1;
            end;

         while l1<l2 do
            begin
               s1 :='0'+s1;{pad with leading zeros}
               l1 := l1+1;
            end;

         while l2<l1 do
            begin
               s2 := '0'+s2;
               l2 := l2+1;
            end;

         if (l1=0) or (l2=0) then
            begin
               minus :='bad';
               exit;
            end;

         if s1<s2 then
            begin
               temp := s1;
               s1 := s2;
               s2 := temp;
               switched := true;
            end;

         place(s1,t1,l1);{place string in math array}
         place(s2,t2,l2);
         cnt :=255;

         l1 := ((l1 -1) div 4) + 1;{2.01 change}

         while (255 - l1) < cnt do
            begin
               if t1[cnt] < t2[cnt] then
                  begin
                     borrow := cnt -1;
                     while (t1[borrow] = 0) and(not borrowed) do
                        begin
                           if borrow=0 then
                              begin
                                 minus :='bad';
                                 exit;
                              end;
                              if t1[borrow]<>0 then
                                 borrowed := true;

                           borrow := borrow -1;
                        end;{while}
                     while borrow < cnt do
                        begin
                           t1[borrow] := t1[borrow] -1;
                           t1[borrow+1] := t1[borrow+1] +big;{2.01 change}
                           borrow := borrow +1;
                        end;{while}
               end;{if}

               r[cnt] := t1[cnt] - t2[cnt];
               cnt := cnt -1;
            end;{while}

         cnt := cnt +1;

         temp := adjust(r,cnt,0);
         while length(temp) < length(s1) do
            temp := '0' + temp;{2.01 add. Grouping consideration for #'s 4 digits or less.}
         if dpos1 > 0 then
            begin
               if length(temp) = 255 then
                  begin
                     minus := 'bad';
                     exit;
                  end
               else
                  temp := copy(temp,1,dpos1-1) + '.' + copy(temp,dpos1,length(temp));
            end;
         if temp[1] = '0' then
            offleadzero(temp);{remove leading zeroes}
         if (pos('.',temp)>0) and (temp[length(temp)]='0') then
            offtrailzero(temp);
         if switched then
            begin
               if spos1=0 then
                  begin
                     if (length(temp)=255) and (pos('.',temp)=0) then
                        begin
                           minus :='bad';
                           exit;
                        end
                     else
                        begin
                           minus :='-'+temp;
                           exit;
                        end;
                  end
               else
                  begin
                     minus := temp;
                     exit;
                  end;
            end
         else
            begin
               if spos1=1 then
                  begin
                     if (length(temp)=255) and (pos('.',temp)=0) then
                        begin
                           minus :='bad';
                           exit;
                        end
                     else
                        minus := '-'+temp;
                  end
               else
                  minus := temp;
            end;{if}
end;{minus}

function times(s1,s2: string; data_ok: boolean): string;
   var
      totshift: integer;
      product: string;
      idx,idx2,le1,le2,cnt2,dp1,dp2,sp1,sp2,dp: byte;
      totoff: integer; {2.01 addition.}
      chkstring: string[4];{2.01 addition, for double length precision.}
      truelength: integer;{2.01--fix of old 'bad' return valus.}
      t1,t2,r: anarray;

   begin
      if not data_ok then
         begin
            validstring := verify(s1);
            if not validstring then
               begin
                  times :='bad';
                  exit;
               end;
            validstring := verify(s2);
            if not validstring then
               begin
                  times :='bad';
                  exit;
               end;

         end;

      totoff := 0;{Version 2.01 addition.}
      truelength := 0;
      if (pos('.',s1)>0) and (s1[length(s1)]='0') then
         offtrailzero(s1);
      if (pos('.',s2)>0) and (s2[length(s2)]='0') then
         offtrailzero(s2);
      if pos('.',s1)=length(s1) then
         strip(s1);{Version 2.01 addition}
      if pos('.',s2)=length(s2) then
         strip(s2);{Version 2.01 addition}
      le1 := length(s1);
      le2 := length(s2);
      sp1 := pos('-',s1);
      sp2 := pos('-',s2);
      dp1 := pos('.',s1);
      dp2 := pos('.',s2);
      if (dp1>0) and (dp1<le1) then
         dp1 := le1-dp1
      else
         dp1 := 0;
      if (dp2>0) and (dp2<le2) then
         dp2 := le2-dp2
      else
         dp2 := 0;

{      if dp1 >0 then
         totlength := totlength -1;} {you don't count decimal point in length}
{      if dp2>0 then
         totlength := totlength -1;}

      totshift := dp1+dp2;

      {if totshift>255 then}
         {begin}
            {times :='bad';}
            {exit;}{Version 2.01, fixed this illogical question.}
            {More accuracy is maintained with new method, even if result is '0'}
         {end;}


      if s1[1] = '0' then
         offleadzero(s1);
      if s2[1] = '0' then
         offleadzero(s2);

   if (s1='0') or (s2='0') then
      begin
         times :='0';
         exit;
      end;
   if (s1='1') or (s2='1') then
      begin
         if s1='1' then
            times :=s2;
         if s2='1' then
            times :=s1;
         exit;
      end;
   if pos('.',s1)>0 then
      begin
         strip(s1);
         offleadzero(s1);
      end;
      if pos('.',s2)>0 then
         begin
            strip(s2);
            offleadzero(s2);
         end;

    while (length(s1) >1) and (s1[length(s1)] ='0') do
       begin
          s1[0] := chr(ord(s1[0]) -1);
          totoff := totoff + 1;       {v. 2.01 addition.}
       end;{trailing zeros are not significant in the "real" prodect.}

    while (length(s2)>1) and (s2[length(s2)] = '0') do
       begin
          s2[0] := chr(ord(s2[0]) -1);
          totoff := totoff + 1;       {v. 2.01 addition.}
       end;{trailing zeros are not significant in the "real" prodect.}

    le1 := length(s1);
    le2 := length(s2);
    if sp1>0 then
       begin
          s1 := copy(s1,2,le1);
          le1 := le1-1;
       end;

    if sp2 >0 then
       begin
          s2 := copy(s2,2,le2);
          le2 := le2-1;
       end;

      if s2>s1 then
      begin
         temp := s1;
         cnt := le1;
         s1 := s2;
         le1 := le2;
         s2 := temp;
         le2 := cnt;
      end;

{      totlength := le1 + le2;}

{      if totlength > 255 then
         begin
            times := 'bad';
            exit;
         end;} {totlength removed as of version 2.01, caused bad 'bad's}

      place(s1,t1,le1);
      place(s2,t2,le2);

      cnt2 := 255;

      idx := 0;

      le2 :=((le2 - 1) div 4) + 1;
      le1 :=((le1 - 1) div 4) + 1;{Version 2.01 adjustment}

      for cnt := 128 to 255 do
         r[cnt] := 0;
      while 255-le2 <cnt2 do
         begin
            cnt := 255;

            idx2 := 255 - idx;            carry := 0;
            while 255-le1<cnt do
               begin
                  r[idx2] := t1[cnt]*t2[cnt2]+carry+r[idx2];
                  carry :=0;
                  if r[idx2]>= big then
                     begin                   {Big is v. 2.01 addition}
                        carry := r[idx2] div big;
                        r[idx2] := r[idx2] - carry*big;
                     end; {if}
               cnt := cnt -1;

               idx2 := idx2 - 1;
            end;{inner while}

         if carry >0 then
            begin
               {if idx2>=192 then}
                  r[idx2] := carry
               {else}
                  {begin}
                     {times :='bad';} {Version 2.01 addition}
                     {exit;}          {Did you carry out of array?}
                  {end;}
            end
         else
            cnt := cnt + 1;
         cnt2 := cnt2 -1;

         idx := idx + 1;
      end;{while}

      if r[idx2] = 0 then
         idx2 := idx2 + 1;{2.01 adjust idx2 after loop.}

      product := adjust(r,idx2,0);
      offleadzero(product);{Version 1.01 addition.  Useless 'bad' avoided.}
      while totoff >0 do {v. 2.01 addition--Put back zeros you took off.}
         begin
            if (length(product) = 255) and (totshift=0) then
               begin
                  times := 'bad';
                  exit; {Product is to big for string.}
               end
            else
               begin
                  if totshift = 0 then
                     product := product + '0'
                  else
                     totshift := totshift - 1;{2.01 move decimal point to the right.}
                  totoff := totoff - 1;
               end;
         end;

   truelength := truelength + ((255 - idx2 + 1) * 4);
   str(r[idx2],chkstring);
   if length(chkstring)< 4 then
      truelength := truelength - (4 - length(chkstring));{2.01}
      {How many chars does array grouped in 4 occupy?}

      if totshift = 0 then
         begin
            if truelength >255 then
               begin
                  times := 'bad';
                  exit;{2.01 Number to big for string!}
               end;
         end
      else
         begin
            if (totshift < truelength) and (truelength >255) then
               begin
                  times := 'bad';
                  exit;
               end;{Also can't fit into string.}
         end;

      while (truelength < totshift) and(totshift>0) do
         begin
            {if length(product)=255 then}
               {begin}
                  {times :='bad';}
                  {exit;}
               {end}
            {else} {2.01 deletion. Accuracy is enhanced like a calculator}
               product :='0'+product;
               truelength := truelength + 1;{Loop exit break.}
         end;{while}

      if totshift = length(product) then
         begin
            {if length(product)=255 then}
               {begin}
                  {times :='bad';}
                  {exit;}
               {end}
            {else}
               {begin} {2.01 deletion to give symetric accuracy.}
                  if (totshift>0) and (totshift=length(product)) then
                     product :='.'+product;
               {end;}
         end
      else
         begin
            if totshift>0 then
               begin
                  {if length(product)=255 then}
                     {begin}
                        {times :='bad';}
                        {exit;}
                     {end}
                  {else} {2.01 deletion. Many calculators "chop" to stay accurate.}
                     product := copy(product,1,length(product)-totshift)
                    +'.'+copy(product,length(product)-totshift+1,length(product));
               end;
         end;


      if product[1] = '0' then
         offleadzero(product);
      if (sp1 xor sp2)=1 then
         begin
            if (length(product)=255) and (pos('.',product)=0) then
               begin {2.01 fix, put sign back ONLY if you lose decimal}
                 times :='bad';{point accuracy, but not integer accuracy.}
                 exit;
            end
         else
            product := '-'+product;
      end;
  times := product;
end;{times}

function divide(s1,s2: string; data_ok,no_decimals: boolean): string;

   var
      resultneg: boolean;
      finished:  boolean;
      result:    string;
      take :     string;
      s3:        string;
      sum:       string;
      remainder: string;
      idx:       byte;
      max:       byte;
      dp1,dp2:   byte;
      sp1,sp2:   byte;
      le1,le2,lr,lr3,top,bottom,middle:   byte;
      track:     array[1..9] of string;
   begin

      if not data_ok then
         begin
            validstring := verify(s1);
            if not validstring then
               begin
                  divide := 'bad';
                  exit;
               end;

            validstring := verify(s2);
            if not validstring then
               begin
                  divide := 'bad';
                  exit;
               end;
         end;

      if (pos('-',s1)>0) xor (pos('-',s2)>0) then
         resultneg := true
      else
         resultneg := false;

      s3 := s2;
      s2 := s1;
      s1 := s3; {switch so user can enter divide(s1,s2) }
      {divide s1 by s2, s1/s2 to keep with math convention}

      sp1 := pos('-',s1);
      sp2 := pos('-',s2);
      if sp1 = 1 then
         s1 := copy(s1,2,length(s1));{chop signs off}

      if sp2 = 1 then
         s2 := copy(s2,2,length(s2));{chop signs off}


         if (pos('.',s1)>0) and (s1[length(s1)]='0') then
            offtrailzero(s1);{remove trailing zeroes if decimaled # }
         if (pos('.',s2)>0) and (s2[length(s2)]='0') then
            offtrailzero(s2);{same for s2}
         if s1[1] = '0' then
            offleadzero(s1);{remove leading zeroes}
         if s2[1] = '0' then
            offleadzero(s2);{also for s2}

         if (s1='0') or (s1='0.') then
            begin
               divide :='bad';
               exit; {division by zero is not allowed}
            end;

         if pos('.',s1)=0 then
            begin
               {if length(s1)=255 then
                  begin
                     divide := 'bad';
                     exit;
                  end;}{2.02 fix.}
               s1 := s1+'.';
            end;

         if pos('.',s2)=0 then
            begin
               {if length(s2)=255 then
                  begin
                     divide := 'bad';
                     exit;
                  end;}{2.02 fix.}
               s2 := s2+'.';
            end;

         if s1=s2 then {s1/s2=1}
            begin
               if resultneg then {is s1/-s2 or -s1/s2, result=-1}
                  divide := '-1'
               else
                  divide := '1';
               exit;
            end;

         if (s2='0') or (s2='0.') then
            begin
               divide :='0';
               exit; {anything into zero gives zero');
            end;

         if (s1='1') or (s1='1.') then
            begin
               if resultneg then
                  divide := '-'+s2
               else
                  divide := s2;
               exit; {dividing anything by 1 gives s2 or negative s2}
            end;

         while pos('.',s1)<length(s1) do
            begin
               if (pos('.',s1)=0) or (pos('.',s2)=0) then
                  begin
                     divide := 'bad';{2.02 fix.}
                     exit;
                  end;

               if pos('.',s1)=1 then
                  s1 := s1[2]+'.'+copy(s1,3,length(s1))
               else
                  if pos('.',s1)=length(s1) then
                     s1 := copy(s1,1,length(s1)-1)+'0'+'.'
                  else
                     s1 := copy(s1,1,pos('.',s1)-1)+s1[pos('.',s1)+1]+'.'+copy(s1,pos('.',s1)+2,length(s1));

               if pos('.',s2)=1 then
                  s2 := s2[2]+'.'+copy(s2,3,length(s2))
               else
                  if pos('.',s2)=length(s2) then
                     s2 := copy(s2,1,length(s2)-1)+'0'+'.'
                  else
                     s2 := copy(s2,1,pos('.',s2)-1)+s2[pos('.',s2)+1]+'.'+copy(s2,pos('.',s2)+2,length(s2));
            end;{while}

            dp1 := pos('.',s1);
            dp2 := pos('.',s2);
            if s1[1] = '0' then
               offleadzero(s1);
            if s2[1] = '0' then
               offleadzero(s2);
            s3 := s2;
            strip(s1);
            strip(s2);

            le1 := length(s1);
            le2 := length(s2);
            result := '';

            if s1 = '1' then
               begin
                  if resultneg then
                     begin
                        divide :='-'+s3;
                        exit;
                     end
                  else
                     begin
                        divide := s3;
                        exit;
                     end;
               end;


            if s1='0' then
               begin
                  divide :='bad';
                  exit;
               end;

            if s1=s2 then
               begin
                  divide :='1';
                  exit;
                end;

            if s2='0' then
               begin
                  divide :='0';
                  exit;
               end;

      remainder := '';

      if resultneg then
         max := 254
      else
         max := 255;

      for idx := 1 to le2 do
         begin
            take := s2[idx];

            if idx = dp2 then
               begin
                  if no_decimals then
                     begin
                        if result[1] = '0' then
                           offleadzero(result);

                        if resultneg then
                           divide := '-'+result
                        else
                           divide := result;
                        exit;
                     end
                  else
                     result := result + '.';
               end;

            remainder := remainder + take;
            lr := length(remainder);

            if lr < le1 then
               result := result + '0';

            if (lr = le1) and (remainder < s1) then
               result := result + '0';

            if ((lr = le1) and (remainder >=s1)) or (le1 <lr) then
               begin
                  top := 9;
                  bottom := 1;
                  middle := (top + bottom) div 2;
                  while top >= bottom do
                     begin
                        str(middle,sum);
                        s3 := times(s1,sum,true);
                        track[middle] := s3;
                        lr3 := length(s3);
                        if ((lr3 = lr) and (s3 > remainder)) or (lr3 > lr ) then
                           begin
                              top := middle - 1;
                              middle := (top + bottom) div 2;
                           end
                        else
                           begin
                              if ((lr3 = lr) and (s3 < remainder)) or (lr3 < lr) then
                                 begin
                                    bottom := middle + 1;
                                    middle := (top + bottom) div 2;
                                 end
                              else
                                 top := bottom-1;
                           end;
                     end; {while}
                  str(middle,sum);
                  remainder := minus(remainder,track[middle],true);
                  result := result + sum;
               end; {if}

            if remainder = '0' then
               remainder := '';

         end; {for}

       if result[1] = '0' then
          offleadzero(result);
       take := '0';
       if remainder <> '' then
          begin
             if result = '0' then
                result := '';

             if pos('.',result) = 0 then
                begin
                   if no_decimals then
                      begin
                         if result = '' then
                            result := '0';

                         if resultneg then
                            divide :='-'+result
                         else
                            divide := result;
                         exit;
                      end
                   else
                      result := result + '.';
                end;

             if (length(result) = max ) or (remainder ='') then
                finished := true
             else
                finished := false;

             while not finished do
                begin

                   remainder := remainder + take;
                   lr := length(remainder);

                   if lr < le1 then
                      result := result + '0';

                   if (lr = le1) and (remainder < s1) then
                      result := result + '0';

                   if ((lr = le1) and (remainder >=s1)) or (le1 <lr) then
                      begin
                         top := 9;
                         bottom := 1;
                         middle := (top + bottom) div 2;
                         while top >= bottom do
                            begin
                               str(middle,sum);
                               s3 := times(s1,sum,true);
                               track[middle] := s3;
                               lr3 := length(s3);
                               if ((lr3 = lr) and (s3 > remainder)) or (lr3 > lr) then
                                  begin
                                     top := middle - 1;
                                     middle := (top + bottom) div 2;
                                  end
                               else
                                  begin
                                     if ((lr3 = lr) and (s3 < remainder)) or (lr3 < lr) then
                                        begin
                                           bottom := middle + 1;
                                           middle := (top + bottom) div 2;
                                        end
                                     else
                                        top := bottom-1;
                                  end;
                            end; {while}
                         str(middle,sum);
                         remainder := minus(remainder,track[middle],true);
                         result := result + sum;
                      end; {if}

                   if remainder = '0' then
                      remainder := '';

                   if (length(result) = max ) or (remainder ='') then
                      finished := true;

                end; {while}

             if resultneg then
                result := '-' + result;

         end;{if}

      divide := result;

   end;{divide}

function fac(s1: string): string;
   var
      i,j,chk: integer;
      n: string;
   begin
      validstring := verify(s1);
      if s1[1] = '0' then
         offleadzero(s1);
      if (s1<'0') or (pos('.',s1)>0) or(not validstring) then
         begin
            fac :='bad';
            exit;
         end
      else
         begin
            if (s1 ='0') or(s1='1') then
               fac := '1'
            else
               begin
                  val(s1,i,chk);
                  if i>145 then
                     begin
                        fac :='bad';
                        exit;
                     end;


                 n :='1';
                 for j := 2 to i do
                    begin
                       str(j,s1);
                       n := times(n,s1,true);
                    end;
                 fac := n;
               end;
         end;
  end;{fac}
function square(s1: string): string;
   begin
     square := times(s1,s1,false);
   end; {square}

function root(s1: string): string;
  {this procedure will find the principal square root of s1}

   var
      dp,idx,endbound: word;
      trial,trial2,result,remainder,group,divide_by: string;
   begin
      validstring := verify(s1);
      if not validstring then
         begin
            root := 'bad';
            exit;
         end;
      if pos('-',s1) >0 then
         begin
            root := 'bad'; {square root of a negative # is erroneous }
            exit;
         end;

   offleadzero(s1);
   offtrailzero(s1);
   if (s1='0') or (s1='0.') or (s1='1') or (s1='1.') then
      begin
         root := s1;
         exit
      end;

   dp := pos('.',s1);
   if dp = 0 then
      begin
         s1 := s1 + '.';
         dp := pos('.',s1);
      end;

   if (dp>0) and (dp<255) then
      begin
         temp := copy(s1,dp+1,length(s1));
         s1 := copy(s1,1,dp-1);
         l1 := length(temp);
         if (l1 div 2) * 2 < l1 then
            temp := temp + '0'; {be sure even # of zeroes for grouping }
      end;

   s1 := s1+temp; {new value with no decimal point}
   endbound := length(s1)+1; {end boundary of # to find root of}

   if dp >0 then
      begin
         while length(s1) < 254 do
            s1 := s1 + '00'; {add zeroes for maximum decimal accuracy}
      end;

   if dp >0 then
      dp := (dp div 2) +1
   else
      dp := 129; {fix decimal allignment for result }

   result :='';
   remainder :='';
   group :='';
   l1 := (length(s1) - ((length(s1) div 2) * 2));
   if l1 = 1 then
      idx := 2
   else
      idx := 3;
   group := copy(s1,1,idx-1); {get first group for square root algorithm}

   {the rest of the routine is the "heart" of the algorithm.  it gets groups}
   {and finds the square root }

   while (group = '00' ) or (group = '0' ) do
      begin
         result := result + '0'; {groups of zero will give result of zero}
         group := copy(s1,idx,2);
         idx := idx + 2;     {prepare for next group to take }
      end;

   offleadzero(group); {ensure group is not 01, 02, 03...etc}

   if length(group) = 1 then
      begin
         if group < '4' then
            begin
               result := result + '1';
               remainder := minus(group,'1',true);
               divide_by := '2'; {largest square into group is 1}
            end
         else
         if (group >='4') and (group <'9') then
            begin
               result := result + '2';
               remainder := minus(group,'4',true); {largest square into group is 2}
               divide_by := '4';
            end
         else
         if group = '9' then
            begin
               result := result + '3';
               remainder := minus(group,'9',true);{largest square is 3}
               divide_by := '6';
            end;
      end
   else
      begin
         if (group >='10') and (group <'16') then
            begin
               result := result + '3';
               remainder := minus(group,'9',true);{other cases of 3^2}
               divide_by := '6';
            end
         else
         if (group >='16') and (group <'25') then
            begin
               result := result + '4';
               remainder := minus(group,'16',true); { 4^2}
               divide_by := '8';
            end
         else
         if (group >='25') and (group <'36') then
            begin
               result := result + '5';
               remainder := minus(group,'25',true); {five squared}
               divide_by := '10';
            end
         else
         if (group >='36') and (group <'49') then
            begin
               result := result + '6';
               remainder := minus(group,'36',true); {etc... ...}
               divide_by := '12';
            end
         else
         if (group >='49') and (group <'64') then
            begin
               result := result + '7';
               remainder := minus(group,'49',true);
               divide_by := '14';
            end
         else
         if (group >='64') and (group <'81') then
            begin
               result := result + '8';
               remainder := minus(group,'64',true);
               divide_by := '16';
            end
         else
            begin
               result := result + '9';
               remainder := minus(group,'81',true);
               divide_by := '18';
            end;
      end;{if}
   group := copy(s1,idx,2); {new group for "heart" of program }
   if remainder = '0' then
      remainder := '';
   remainder := remainder + group;
   idx := idx + 2;

   while (idx < 255) do {for as many groups as there are do & not a perfect square}
      begin
         if length(divide_by)+1 > length(remainder) then
            trial := '0'
         else
            trial := divide(remainder,divide_by+'0',true,true);
         if trial = '0' then
            divide_by := divide_by+'0'
         else
            begin
               trial2 :='';
               while (trial2 <> trial) do
                  begin
                     trial2 := trial;
                     if length(trial) >1 then
                        trial := '9' {greater than 10? adjust to range 1-9}
                     else
                        trial := divide(remainder,divide_by+trial,true,true);
                     if minus(trial,'1',true) = trial2 then
                        begin
                           if length(trial) >1 then
                              trial := '9'
                           else
                              trial := divide(remainder,divide_by+trial,true,true);
                        end;
                  end;
               divide_by := divide_by + trial;
               remainder := minus(remainder,times(divide_by,trial,true),true);
               divide_by := plus(divide_by,trial,true);
            end;

         group := copy(s1,idx,2); {new group for next pass}
         idx := idx + 2;
         result := result + trial;
         if remainder = '0' then
            remainder := '';
         remainder := remainder + group;
      end;{while--main square root finder}

      result := copy(result,1,dp-1) +'.'+ copy(result,dp,length(result));
      offtrailzero(result);
      root := result;
end; {root}
end.{unit}