/****************************************************************************\

				Save Parameters  Version 1.03  by Douglas Peterson

	Revisions:

		1.00	First Release

		1.01	Fixed bug that added new bytes to end of .EXE every time
				 you called Save_Parameters()

		1.02  It didn't add new bytes, but you could only save once :)
				 Oh well, it's fixed now :)  ("r+b" NOT "a+b")

		1.03  No longer changes date of .EXE file when saving.


 ****************************************************************************

	This is a header file will allow you to save data to you projects .EXE
file.  This will only work with Borland/Turbo C/C++ and has only been
tested under Version 3.1.  If you get a linker error:  Undefined symbol
*_C0argv in module xxx, you will NOT be able to use the file without
modifying it.

	Edit the typedef structure SAVE_STRUCT to include the elements you want
saved in the .EXE file.

	Use ONLY static data, don't use pointers!

	If you want to call the structure something other than "save", edit the
SAVE_STRUCT_NAME macro.

	As of version 1.03 the files date and time will not be altered by the
update to the .EXE file.  This code uses getftime() and setftime() as
defined in the IO.H header file.  If you do not want the date/time preserved,
rem out the #define RESTORE_TIME macro definition.

	To save changes to the structure in the .EXE, call Save_Parameters().
The saved values will be loaded up and ready to go just before main() is
called.

NOTE:  If you want to load the parameters sometime AFTER main() is called,
you can remove the #pragma startup directive later in this file, and call
Load_Parameters() whenever nessasary.  This might be usefull if you wanted
to have a command line switch to 'restore defaults'.  In this case you
would simply call Save_Parameters() INSTEAD of Load_Parameters() after
parsing the command line.

	Save_Parameters() returns:

		0 if successfull
		1 if the file could not be opened properly
		2 if the file could not be written to

	This version uses getftime() and setftime() to prevent the file date
and time from being altered when Save_Parameters() is called.  If you don't
want this option or are unable to compile it with these functions, remove
the macro declaration: #define RESTORE_TIME later in this file.


 ****************************************************************************

!!!!NOTE!!!!

	You will need main to accept at least argc and argv.
void main(int argc, char **argv)  Call them what you like, but if you don't
include them as parameters to main, the C startup code gets linked into
into your program will NOT read the PSP for the info save_parameters() needs.

	It doesn't matter what you call **argv because save_parameters() uses
the global **_C0argv which is the global version of the pointer that gets
passed to main.

	You will need a separate copy of this file for EACH project you want to
use these functions in, as each copy must be edited specifically for each
project.

	Each time your program is linked, the saved data will be lost.  You can
save data even if your .EXE file has the symbol table in it.  If you save
data to the .EXE file with symbol data in it and then run TDSTRIP or similar
symbol table stripper, the saved data will be lost, but no harm will be done
to the .EXE file.

	If after saving data into the .EXE the program loads without loading the
saved data:

	1) Check to see that your main() declaration includes argc and argv
	2) Check the #pragma startup directive later in this file.  The
priority number may be conflicting with another module in your project.

	DO NOT ALTER argv[0]!!  Save_Parameters() uses this variable to find the
path and name of the .EXE file at run time.  You should always copy the
contents of argv[0] to a static buffer before altering it anyhow as if you
add bytes to it you will overwrite something else.


 ****************************************************************************


	This file is freeware and I can not and will not be held responsible
for any problems that occur coincidenally or otherwise with the use of
this code.

	If you have any questions, comments, would like to report a bug, or
can help me modify this to support more compilers, please write to me
at:


	Douglas Peterson
	P.O. Box 26331
	San Diego, CA  92196

	-or-

	Compuserve  73014,3320

\****************************************************************************/

#define RESTORE_TIME				//If defined, file time/date won't be altered

typedef struct {					//Put the data types you want saved here
	int  num;	//..example delete me
	char a[128];//..example delete me
	char b[128];//..example delete me
	char c[128];//..example delete me
}SAVE_STRUCT;

#define SAVE_STRUCT_NAME save		//Name you want to use for save structure
SAVE_STRUCT SAVE_STRUCT_NAME={	//Initialize your data here with default values
33,                            //..example delete me
"This is the A default string",//..example delete me
"This is the B default string",//..example delete me
"This is the C default string" //..example delete me
};



/****************************************************************************\

							INCLUDES AND FUNCTION DEFINITIONS

\****************************************************************************/
#include <stdio.h>
#include <string.h>

#ifdef RESTORE_TIME
#include <io.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif
void Load_Parameters(void);
int  Save_Parameters(void);
#ifdef __cplusplus
}
#endif

#pragma startup Load_Parameters 249	//If you need to change priority
												//See #pragma startup in C user manual


/****************************************************************************\

												DATA

\****************************************************************************/
#define SAVE_MARKER "HeAdEr_MaRk" //12 bytes
#define SAVE_HEADER_SIZE 12       //
#define SAVE_OFFSET ((long)-((long)sizeof(SAVE_STRUCT)+(long)SAVE_HEADER_SIZE))
#define SAVE_OFFSETA ((long)-((long)sizeof(SAVE_STRUCT)))
#define FILE_ERROR  "Error opening file -> %s"

extern char **_C0argv;	//Name and path of .EXE even if user renamed it

#define EXEPATHNAME *_C0argv

#ifdef RESTORE_TIME
static struct ftime ft;
#endif

/****************************************************************************\


\****************************************************************************/
/****************************************************************************\

	void Load_Parameters(void)

\****************************************************************************/
void Load_Parameters(void)
{

	FILE *fp;
	char temp[SAVE_HEADER_SIZE]="";

	if ((fp=fopen(EXEPATHNAME,"rb"))==NULL) return;
#ifdef RESTORE_TIME
	getftime(fileno(fp), &ft);
#endif
	fseek(fp,SAVE_OFFSET,SEEK_END);
	fread(temp,SAVE_HEADER_SIZE,1,fp);
	if (strcmp(temp,SAVE_MARKER)!=0) { fclose(fp); return; }
	fread(&SAVE_STRUCT_NAME,sizeof(SAVE_STRUCT),1,fp);
	fclose(fp);

}

/****************************************************************************\

	int Save_Parameters(void)

		Saves the contents of the SAVE_STRUCT to the end of the .EXE file, no
	matter what the file has been named or renamed.

	returns: 0 if successfull
				1 if the file could not be opened properly
				2 if the file could not be written to

\****************************************************************************/
int Save_Parameters(void)
{

	FILE *fp;
	char temp[SAVE_HEADER_SIZE]="";

	if ((fp=fopen(EXEPATHNAME,"r+b"))==NULL) return(1);
	fseek(fp,SAVE_OFFSET,SEEK_END);
	fread(temp,SAVE_HEADER_SIZE,1,fp);

	if (strcmp(temp,SAVE_MARKER)!=0) {
		fseek(fp,0L,SEEK_END);
		if (fwrite(SAVE_MARKER,SAVE_HEADER_SIZE,1,fp)!=1) return(2);
	}
	else fseek(fp,SAVE_OFFSETA,SEEK_END);

	if (fwrite(&SAVE_STRUCT_NAME,sizeof(SAVE_STRUCT),1,fp)!=1) return(2);
#ifdef RESTORE_TIME
	fflush(fp);
	setftime(fileno(fp), &ft);
#endif
	fclose(fp);

	return(0);

}

